#include <ell/CS_LP_PivotM.hh>
#include <fstream>	// for readBinaryInputClone(..), writeBinaryOutput(..)

namespace ell
{
	CS_LP_PivotM::CS_LP_PivotM(
	 const biu::LatticeModel* _lattice, const biu::ContactEnergyFunction* _energy,
	 const biu::Sequence& seq, const biu::Structure& struc)
	 : seqLength(seq.size()), lattice(_lattice), energy(_energy),
	   cSequence(compressSeq(seq, energy->getAlphabet())),
	   cStructure(compressSeq(struc, lattice->getDescriptor()->getAlphabet()))
	{}

	CS_LP_PivotM::~CS_LP_PivotM() {
	}

	std::string
	CS_LP_PivotM::getUncompressedSeqStr() const {
		return uncompressSeq(cSequence, energy->getAlphabet(),
							   seqLength);
	}

	std::string
	CS_LP_PivotM::getUncompressedStructStr() const {
			// structure string holds one character less than sequence string
		return uncompressSeq(cStructure, lattice->getDescriptor()
							   ->getAlphabet(), seqLength-1);
	}

	bool	
	CS_LP_PivotM::operator== (const CState& _cState2) const {
		CS_LP_PivotM cState2 =
		 dynamic_cast<const CS_LP_PivotM&>(_cState2);
		return (this->seqLength == cState2.seqLength
				  && *this->lattice == *cState2.lattice
				  && *this->energy == *cState2.energy
				  && this->cSequence == cState2.cSequence
				  && this->cStructure == cState2.cStructure);
	}

	bool
	CS_LP_PivotM::operator!= (const CState& _cState2) const {
		CS_LP_PivotM cState2 =
		 dynamic_cast<const CS_LP_PivotM&>(_cState2);
		return (this->seqLength != cState2.seqLength
				  && *this->lattice != *cState2.lattice
				  && *this->energy != *cState2.energy
				  && this->cSequence != cState2.cSequence
				  && this->cStructure != cState2.cStructure);
	}

	CS_LP_PivotM* 
	CS_LP_PivotM::clone() const {
		return new CS_LP_PivotM(*this);
	}

	std::string
	CS_LP_PivotM::getCStateStringRepresentation() const {
		std::string strRepr;
		for (size_t i = 0; i < cSequence.size(); i++) {
			strRepr += cSequence[i];
		}
		for (size_t i = 0; i < cStructure.size(); i++) {
			strRepr += cStructure[i];
		}
		return strRepr;
	}

	std::string
	CS_LP_PivotM::getUncomprStateStringRepresentation()
	 const {
		return ( getUncompressedStructStr() + "(" +
				  getUncompressedSeqStr() +")" );
	}

	State*
	CS_LP_PivotM::uncompressedClone() const {
		return new S_LP_PivotM(lattice, energy,
					 getUncompressedSeqStr(), getUncompressedStructStr(), false);
	}

	void
	CS_LP_PivotM::uncompress(State& state) const {
		dynamic_cast<S_LP_PivotM&>(state) =
		 S_LP_PivotM(lattice, energy,
		  getUncompressedSeqStr(), getUncompressedStructStr(), false);
	}

	CS_LP_PivotM*
	CS_LP_PivotM::readBinaryInputClone(std::ifstream& source)
	 const {

			// create new CS_LP_PivotM
		CS_LP_PivotM* cState =
		 new CS_LP_PivotM();

			// get lattice model and energy function from the dummy state
			// which calls this function
		cState->lattice = this->lattice;
		cState->energy = this->energy;

			// read seqLength
		source.read(reinterpret_cast<char*>(&(cState->seqLength)),
						   sizeof(cState->seqLength));

		size_t size;

			// read cSequence
		source.read(reinterpret_cast<char*>(&size), sizeof(size));
		cState->cSequence.resize(size);
		for (size_t i = 0; i < cState->cSequence.size(); i++) {
			source.read(reinterpret_cast<char*>(&(cState->cSequence[i])),
						  sizeof(cState->cSequence[i]));
		}

			// read cStructure
		source.read(reinterpret_cast<char*>(&size), sizeof(size));
		cState->cStructure.resize(size);
		for (size_t i = 0; i < cState->cStructure.size(); i++) {
			source.read(reinterpret_cast<char*>(&(cState->cStructure[i])),
						  sizeof(cState->cStructure[i]));
		}

		return cState;
	}

	void
	CS_LP_PivotM::writeBinaryOutput(std::ofstream& destination)
	 const {
			// write seqLength
		destination.write(
	 	 reinterpret_cast<const char*>(&seqLength), sizeof(seqLength));

		size_t size;

			// write cSequence
		size = cSequence.size();
		destination.write(reinterpret_cast<const char*>(&size), sizeof(size));
		for (size_t i = 0; i < size; i++) {
			destination.write(reinterpret_cast<const char*>(&(cSequence[i])),
							   sizeof(cSequence[i]));
		}

			// write cStructure
		size = cStructure.size();
		destination.write(reinterpret_cast<const char*>(&size), sizeof(size));
		for (size_t i = 0; i < size; i++) {
			destination.write(reinterpret_cast<const char*>(&(cStructure[i])),
							   sizeof(cStructure[i]));
		}
	}

} // namespace ell
