#ifndef CLATTICEPROTEINPIVOTMOVESTRUCTSTATE_HH_
#define CLATTICEPROTEINPIVOTMOVESTRUCTSTATE_HH_


#include "CS_LP_PivotM.hh"
#include "S_LP_PivotM_Struct.hh"

namespace ell
{	
		/**
		 * An object of this class represents a lattice protein pivot move
		 * state with a common sequence, contact energy function and
		 * lattice model in a compressed representation.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class CS_LP_PivotM_Struct : public CS_LP_PivotM
	{
	protected:
		CS_LP_PivotM_Struct()
		{}

			/*! Returns the uncompressed common lattice protein sequence
			 *  string.
			 */
		virtual std::string getUncompressedSeqStr() const;

			/*! Returns the uncompressed lattice protein structure string
			 *  (relative moves).
			 */
		virtual std::string getUncompressedStructStr() const;

	public:
		CS_LP_PivotM_Struct(const biu::Structure& struc);

		virtual ~CS_LP_PivotM_Struct();

	// abstract functions (CState)
		virtual bool			operator== (const CState& cState2) const;
		virtual bool			operator!= (const CState& cState2) const;

		virtual CS_LP_PivotM_Struct*	clone() const;

		virtual std::string	getCStateStringRepresentation() const;

			/*! Returns a specific std::string representation of the
			 *  uncompressed representation of this state.
			 * 
			 *  RELATIVEMOVESTRING */
		virtual std::string	getUncomprStateStringRepresentation() const;

		virtual State* uncompressedClone() const;

		virtual void	uncompress(State& state) const;

		virtual CS_LP_PivotM_Struct*
				  readBinaryInputClone(std::ifstream& source) const;

		virtual void	writeBinaryOutput(std::ofstream& destination) const;
	};

}

#endif /*CLATTICEPROTEINPIVOTMOVESTRUCTSTATE_HH_*/
