#include <ell/CS_RNAfe_SingleM.hh>
#include <fstream>	// for readBinaryInputClone(..), writeBinaryOutput(..)

namespace ell
{
	CS_RNAfe_SingleM::CS_RNAfe_SingleM(
	 const biu::Sequence& seq, const biu::Structure& struc)
	 : seqLength(seq.size()),
	   cSequence(compressSeq(seq, RNAFreeEnergy::getSequenceAlphabet())),
	   cStructure(compressSeq(struc, RNAFreeEnergy::getStructureAlphabet())) {
	}

	CS_RNAfe_SingleM::~CS_RNAfe_SingleM() {
	}

	std::string
	CS_RNAfe_SingleM::getUncompressedSeqStr() const {
		return uncompressSeq(cSequence, RNAFreeEnergy::getSequenceAlphabet(),
							   seqLength);
	}

	std::string
	CS_RNAfe_SingleM::getUncompressedStructStr() const {
		return uncompressSeq(cStructure,RNAFreeEnergy::getStructureAlphabet(),
							   seqLength);
	}

	bool	
	CS_RNAfe_SingleM::operator== (const CState& _cState2) const {
		CS_RNAfe_SingleM cState2 =
		 dynamic_cast<const CS_RNAfe_SingleM&>(_cState2);
		return (this->seqLength == cState2.seqLength
				  && this->cSequence == cState2.cSequence
				  && this->cStructure == cState2.cStructure);
	}

	bool
	CS_RNAfe_SingleM::operator!= (const CState& _cState2) const {
		CS_RNAfe_SingleM cState2 =
		 dynamic_cast<const CS_RNAfe_SingleM&>(_cState2);
		return (this->seqLength != cState2.seqLength
				  && this->cSequence != cState2.cSequence
				  && this->cStructure != cState2.cStructure);
	}

	CS_RNAfe_SingleM* 
	CS_RNAfe_SingleM::clone() const {
		return new CS_RNAfe_SingleM(*this);
	}

	std::string
	CS_RNAfe_SingleM::getCStateStringRepresentation() const {
		std::string strRepr;
		for (size_t i = 0; i < cSequence.size(); i++) {
			strRepr += cSequence[i];
		}
		for (size_t i = 0; i < cStructure.size(); i++) {
			strRepr += cStructure[i];
		}
		return strRepr;
	}

	std::string
	CS_RNAfe_SingleM::getUncomprStateStringRepresentation()
	 const {
		return ( getUncompressedStructStr() + "(" +
				  getUncompressedSeqStr() +")" );
	}

	State*
	CS_RNAfe_SingleM::uncompressedClone() const {
		return new S_RNAfe_SingleM(getUncompressedSeqStr(),
												  getUncompressedStructStr());
	}

	void
	CS_RNAfe_SingleM::uncompress(State& state) const {
		dynamic_cast<S_RNAfe_SingleM&>(state) =
		 S_RNAfe_SingleM(getUncompressedSeqStr(),
									  getUncompressedStructStr());
	}

	CS_RNAfe_SingleM*
	CS_RNAfe_SingleM::readBinaryInputClone(std::ifstream& source)
	 const {

			// create new CS_RNAfe_SingleM
		CS_RNAfe_SingleM* cState =
		 new CS_RNAfe_SingleM();

			// read seqLength
		source.read(reinterpret_cast<char*>(&(cState->seqLength)),
						   sizeof(cState->seqLength));

		size_t size;

			// read cSequence
		source.read(reinterpret_cast<char*>(&size), sizeof(size));
		cState->cSequence.resize(size);
		for (size_t i = 0; i < cState->cSequence.size(); i++) {
			source.read(reinterpret_cast<char*>(&(cState->cSequence[i])),
						  sizeof(cState->cSequence[i]));
		}

			// read cStructure
		source.read(reinterpret_cast<char*>(&size), sizeof(size));
		cState->cStructure.resize(size);
		for (size_t i = 0; i < cState->cStructure.size(); i++) {
			source.read(reinterpret_cast<char*>(&(cState->cStructure[i])),
						  sizeof(cState->cStructure[i]));
		}

		return cState;
	}

	void
	CS_RNAfe_SingleM::writeBinaryOutput(std::ofstream& destination)
	 const {
			// write seqLength
		destination.write(
	 	 reinterpret_cast<const char*>(&seqLength), sizeof(seqLength));

		size_t size;

			// write cSequence
		size = cSequence.size();
		destination.write(reinterpret_cast<const char*>(&size), sizeof(size));
		for (size_t i = 0; i < size; i++) {
			destination.write(reinterpret_cast<const char*>(&(cSequence[i])),
							   sizeof(cSequence[i]));
		}

			// write cStructure
		size = cStructure.size();
		destination.write(reinterpret_cast<const char*>(&size), sizeof(size));
		for (size_t i = 0; i < size; i++) {
			destination.write(reinterpret_cast<const char*>(&(cStructure[i])),
							   sizeof(cStructure[i]));
		}
	}

} // namespace ell
