#ifndef CRNAFREEENERGYSINGLEMOVESTATE_HH_
#define CRNAFREEENERGYSINGLEMOVESTATE_HH_


#include "CState.hh"
#include "S_RNAfe_SingleM.hh"

namespace ell
{	
		/**
		 * An object of this class represents a RNA free energy single move
		 * state in a compressed representation.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class CS_RNAfe_SingleM : public CState
	{
	private:
			//! the length of the uncompressed RNA sequence
		size_t seqLength;

			//! the RNA sequence in compressed representation
		std::vector<unsigned char> cSequence;

	protected:
			//! the RNA structure in compressed representation
		std::vector<unsigned char> cStructure;

		CS_RNAfe_SingleM()
		{}

			/*! Returns the uncompressed RNA sequence string.
			 */
		virtual std::string getUncompressedSeqStr() const;

			/*! Returns the uncompressed RNA structure string.
			 */
		virtual std::string getUncompressedStructStr() const;

	public:
		CS_RNAfe_SingleM(const biu::Sequence& seq,
									  const biu::Structure& struc);

		virtual ~CS_RNAfe_SingleM();

	// abstract functions (CState)
		virtual bool			operator== (const CState& cState2) const;
		virtual bool			operator!= (const CState& cState2) const;

		virtual CS_RNAfe_SingleM*	clone() const;

		virtual std::string	getCStateStringRepresentation() const;

			/*! Returns a specific std::string representation of the
			 *  uncompressed representation of this state.
			 * 
			 *  BRACKETNOTATION(SEQUENCE) */
		virtual std::string	getUncomprStateStringRepresentation() const;

		virtual State* uncompressedClone() const;

		virtual void	uncompress(State& state) const;

		virtual CS_RNAfe_SingleM*
				  readBinaryInputClone(std::ifstream& source) const;

		virtual void	writeBinaryOutput(std::ofstream& destination) const;
	};

}

#endif /*CRNAFREEENERGYSINGLEMOVESTATE_HH_*/
