#include <ell/S_LP_PivotM_Struct.hh>
#include <ell/CS_LP_PivotM_Struct.hh>
#include <biu/assertbiu.hh>

namespace ell
{

	const biu::LatticeModel*
	 S_LP_PivotM_Struct::commonLattice = NULL;
	const biu::ContactEnergyFunction*
	 S_LP_PivotM_Struct::commonEnergy = NULL;	
	std::string S_LP_PivotM_Struct::commonSeqStr;
//////////////////////////////////////////////////////////
// S_LP_PivotM_Struct
//////////////////////////////////////////////////////////

	S_LP_PivotM_Struct::S_LP_PivotM_Struct(
	 const std::string& moveString, const bool isAbsoluteMove)
	 : S_LP_PivotM(commonLattice, commonEnergy,
		commonSeqStr, moveString, isAbsoluteMove)
	{}

	S_LP_PivotM_Struct::S_LP_PivotM_Struct(
				const S_LP_PivotM_Struct& latProtPMState2)
	 : S_LP_PivotM(latProtPMState2)
	{}

	S_LP_PivotM_Struct::~S_LP_PivotM_Struct() {
	}

	void
	S_LP_PivotM_Struct::setCommonLatticeModel(
	 const biu::LatticeModel* lattice) {
		assertbiu ( commonLattice == NULL,
		"It is not allowed to set the common lattice model more than once.");
		commonLattice = lattice;
	}

	void
	S_LP_PivotM_Struct::setCommonContactEnergyFunction(
	 const biu::ContactEnergyFunction* energy) {
		assertbiu ( commonEnergy == NULL,
		"It is not allowed to set the common energy function more than once.");
		commonEnergy = energy;
	}

	void
	S_LP_PivotM_Struct::setCommonSequenceString(
	 const std::string& seqStr) {
		assertbiu ( commonSeqStr.empty(),
		"It is not allowed to set the common sequence string more than once.");
		commonSeqStr = seqStr;
	}

	S_LP_PivotM_Struct* 
	S_LP_PivotM_Struct::clone() const {
		return new S_LP_PivotM_Struct(*this);
	}

	CState*
	S_LP_PivotM_Struct::compressedClone() const {
		return new CS_LP_PivotM_Struct(
		 latProt.getStructure());
	}

	void
	S_LP_PivotM_Struct::compress(CState& cState) const {
		dynamic_cast<CS_LP_PivotM_Struct&>(cState)
		 = CS_LP_PivotM_Struct(
			latProt.getStructure());
	}

	State::NeighborListPtr 
	S_LP_PivotM_Struct::getNeighborList() const {
		return NeighborListPtr(new NeighborList(this));
	}

	State::NeighborListPtr
	S_LP_PivotM_Struct::getRandomNeighborList()
				const {
		return NeighborListPtr(new RandomNeighborList(this));
	}

	// NeighborList
	S_LP_PivotM_Struct::NeighborList::NeighborList(
	 			const S_LP_PivotM_Struct* _origin) 
	 : S_LP_PivotM::NeighborList(_origin), origin(_origin)
	{
	}

	State* 
	S_LP_PivotM_Struct::NeighborList::first(
		      State::NeighborList::ItState** itstate) const {
		if (*itstate != NULL) delete *itstate;

		ItState* myitstate = new ItState();
		*itstate = myitstate;

		S_LP_PivotM_Struct* latProtState = NULL;

		PivotMove firstMove = origin->firstPivotMove();

		if (firstMove != PivotMove(0,INVALID_INDEX)) {
			latProtState = new S_LP_PivotM_Struct(*origin);
			latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
			 firstMove.first, firstMove.second);
			myitstate->lastMove = firstMove;
				// if applied pivot move generates an invalid neighbor,
				// call next(..) to iterate neighbors until a VALID one is
				// found which will be returned
			if (!(latProtState->latProt.isValid())) {
				return next(*itstate, latProtState);
			}
		}

		return latProtState;
	}

	State* 
	S_LP_PivotM_Struct::NeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const {
		assertbiu(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);

			// find next move
		PivotMove nextMove = origin->nextPivotMove(myitstate->lastMove);
			// no more moves possible
		if (nextMove == PivotMove(0,INVALID_INDEX)) {
			return NULL;
		}
		else {
			S_LP_PivotM_Struct* latProtState =
			 dynamic_cast<S_LP_PivotM_Struct*>(elem);
				// set latProtState to the state of origin of the neighbors
			latProtState->latProt = origin->latProt;
				// perform next move
			latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
			 nextMove.first, nextMove.second);
			myitstate->lastMove = nextMove;
				// if applied pivot move generates an invalid neighbor,
				// call next(..) to iterate neighbors until a VALID one is
				// found which will be returned
			if (!(latProtState->latProt.isValid())) {
				return next(itstate, latProtState);
			}
			return latProtState;
		}
	}

	// RandomNeighborList
	S_LP_PivotM_Struct::RandomNeighborList::RandomNeighborList(
	 		const S_LP_PivotM_Struct* _origin)
	 : S_LP_PivotM::RandomNeighborList(_origin),
	   origin(_origin) 
	{
	}

	State* 
	S_LP_PivotM_Struct::RandomNeighborList::first(
		      State::NeighborList::ItState** itstate) const {
		if (*itstate != NULL) delete *itstate;

		*itstate = new ItState();
		S_LP_PivotM_Struct* latProtState = 
		 new S_LP_PivotM_Struct(*origin);

		return next(*itstate, latProtState);
	}

	State* 
	S_LP_PivotM_Struct::RandomNeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const {
		assertbiu(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);
		S_LP_PivotM_Struct* latProtState =
		 dynamic_cast<S_LP_PivotM_Struct*>(elem);

		PivotMove nextMove;

			// try to find a valid move at random
		if (myitstate->switchModeValue > 0) {
				// position and index of the next pivot move
			size_t pos, moveIndex;
			bool foundValidMove = false;

			while (!foundValidMove) {
					// move position has to be in [1, structure length)
				pos = (random() % (origin->latProt.getStructureRef().size()-1))+1;
					// moveIndex has to be in [0, number of available moves)
				moveIndex = random() % origin->pivotMoveSet.getMoveNumber();

				nextMove = PivotMove(pos, moveIndex);

					// move has already been chosen
				if (!myitstate->chosenMoves.insert(nextMove).second) {
					continue;
				}
					// move hasn't been chosen yet
				else {
						// set latProtState to the state of origin of the
						// neighbors
					latProtState->latProt = origin->latProt;
					latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
					 nextMove.first, nextMove.second);
						// if applied pivot move generates an invalid neighbor,
						// iterate neighbors until a VALID one is found which
						// will be returned
					foundValidMove = latProtState->latProt.isValid();
					if (foundValidMove) {
						myitstate->switchModeValue--;
					}
				}	
			}
				// enough valid neighbors seen, switching mode to complete
				// valid neighbor enumeration
			if (myitstate->switchModeValue == 0) {
				switchMode(myitstate);
			}
		}
			// chose one of the precalculated valid moves 
		else {
			size_t n = myitstate->unchosenMoves.size();
			if (n == 0) {
				return NULL;
			}
			else {

// @TODO : Rekursion rausnehmen !!!

				size_t i = random() % n;
					// set latProtState to the state of origin of the neighbors
				latProtState->latProt = origin->latProt;
				nextMove = myitstate->unchosenMoves[i];
				latProtState->pivotMoveSet.applyMoveInPlace(
									&latProtState->latProt,
									nextMove.first, nextMove.second);
				myitstate->unchosenMoves[i] = myitstate->unchosenMoves[n-1];
				myitstate->unchosenMoves.resize(n-1);
					// if applied pivot move generates an invalid neighbor,
					// call next(..) to iterate neighbors until a VALID one is
					// found which will be returned
				if (!(latProtState->latProt.isValid())) {
					return next(itstate, latProtState);
				}
			}
		}
		return latProtState;
	}

	State* 
	S_LP_PivotM_Struct::getRandomNeighbor(State* inPlaceNeigh) const {
		S_LP_PivotM_Struct* neigh = NULL;
			// copy this state
		if (inPlaceNeigh != NULL) {
			neigh = dynamic_cast<S_LP_PivotM_Struct*>(inPlaceNeigh);
			assertbiu(neigh != NULL, "inPlaceNeigh is no S_LP_PivotM"); 
			(*neigh) = (*this); 
		} else {
			neigh = new S_LP_PivotM_Struct(*this);
		}

			// position and index of the next pivot move
		size_t pos, moveIndex;

		while (true) {
				// move position has to be in [1, structure length)
			pos = (random() % (latProt.getStructureRef().size()-1))+1;
				// moveIndex has to be in [0, number of available moves)
			moveIndex = random() % pivotMoveSet.getMoveNumber();

				// apply change
			pivotMoveSet.applyMoveInPlace(&neigh->latProt, pos, moveIndex);
				// if applied pivot move generates an invalid neighbor,
				// iterate neighbors until a VALID one is found which
				// will be returned
			if(neigh->latProt.isValid()) {
				return neigh;
			} else {
				// restore lattice protein
				pivotMoveSet.undoLastMove(&neigh->latProt);
			}
		}		
		return NULL;
	}

} // namespace ell
