#include <ell/S_LP_PivotM_StructCUBNorm.hh>
#include <ell/CS_LP_PivotM_StructCUBNorm.hh>
#include <biu/assertbiu.hh>

namespace ell
{

	const biu::LatticeDescriptorCUB
	 S_LP_PivotM_StructCUBNorm::commonCUBLd;
	const biu::LatticeModel
	 S_LP_PivotM_StructCUBNorm::commonCUBLattice(
	  &commonCUBLd);
	const biu::ContactEnergyFunction*
	 S_LP_PivotM_StructCUBNorm::commonEnergy = NULL;	
	std::string S_LP_PivotM_StructCUBNorm::commonSeqStr;
//////////////////////////////////////////////////////////
// S_LP_PivotM_StructCUBNorm
//////////////////////////////////////////////////////////

	S_LP_PivotM_StructCUBNorm::
	 S_LP_PivotM_StructCUBNorm(
	  const std::string& moveString, const bool isAbsoluteMove)
	  : S_LP_PivotM(&commonCUBLattice, commonEnergy,
		 commonSeqStr, moveString, isAbsoluteMove)
	 {}

	S_LP_PivotM_StructCUBNorm::
	 S_LP_PivotM_StructCUBNorm(
	  const S_LP_PivotM_StructCUBNorm& latProtCUBNormState2)
	  : S_LP_PivotM(latProtCUBNormState2)
	 {}

	S_LP_PivotM_StructCUBNorm::
	 ~S_LP_PivotM_StructCUBNorm() {
	}

	bool	
	S_LP_PivotM_StructCUBNorm::
	 operator== (const State& state2) const {
		return normalize() == dynamic_cast<
			const S_LP_PivotM_StructCUBNorm&>(state2).
			 normalize();
	}

	bool
	S_LP_PivotM_StructCUBNorm::
	 operator!= (const State& state2) const {
		return normalize() != dynamic_cast<
			const S_LP_PivotM_StructCUBNorm&>(state2).
			 normalize();
	}

	std::string
	S_LP_PivotM_StructCUBNorm::normalize() const {
		std::string normalizedRelMove = latProt.getRelativeMoveString();

		assertbiu ( normalizedRelMove[0] == 'F',
		 "It's assumed that the relative move string starts with F.");

			// position of char which spans the 2nd and 3rd dimension
		size_t firstOf2ndDim, firstOf3rdDim;

		firstOf2ndDim = normalizedRelMove.find_first_not_of('F', 1);
			// 2nd spanned dimension is L/R and 3rd spanned dimension is U/D
		if (normalizedRelMove[firstOf2ndDim] == 'L'
			 || normalizedRelMove[firstOf2ndDim] == 'R') {
			firstOf3rdDim = normalizedRelMove.find_first_not_of("FLR", 1);
		}
			// 2nd spanned dimension is U/D and 3rd spanned dimension is L/R
		else {
			firstOf3rdDim = normalizedRelMove.find_first_not_of("FUD", 1);
		}

		switch (normalizedRelMove[firstOf2ndDim]) {
			case 'L' :
					// FLD...
				if (normalizedRelMove[firstOf3rdDim] == 'D') {
					break;
				}
					// FLU...
				else {
					swapCharsInString(normalizedRelMove,'L','U','R','D');
					break;
				}
			case 'R' :
					// FRD...
				if (normalizedRelMove[firstOf3rdDim] == 'D') {
					swapCharsInString(normalizedRelMove,'R','D','L','U');
					break;
				}
					// FRU...
				else {
					swapCharsInString(normalizedRelMove,'R','U','L','D');
					break;
				}
			case 'D' :
					// FDL...
				if (normalizedRelMove[firstOf3rdDim] == 'L') {
					swapCharsInString(normalizedRelMove,'D','L','U','R');
					break;
				}
					// FDR...
				else {
					swapCharsInString(normalizedRelMove,'U','L','D','R');
					break;
				}
			case 'U' :
					// FUL...
				if (normalizedRelMove[firstOf3rdDim] == 'L') {
					swapCharsInString(normalizedRelMove,'D','R','U','L');
					break;
				}
					// FUR...
				else {
					swapCharsInString(normalizedRelMove,'U','R','D','L');
					break;
				}
		}

		return normalizedRelMove;
	}

	void
	S_LP_PivotM_StructCUBNorm::swapCharsInString (
	 std::string& s, char l, char d, char r, char u) const {
		for (size_t i = 1; i < s.size(); i++) {
			switch (s[i]) {
				case 'L':
					s[i] = l; break;
				case 'D':
					s[i] = d; break;
				case 'R':
					s[i] = r; break;
				case 'U':
					s[i] = u; break;
				default:
					break;
			}
		}
		return;
	}

	void
	S_LP_PivotM_StructCUBNorm::setCommonContactEnergyFunction(
	 const biu::ContactEnergyFunction* energy) {
		assertbiu ( commonEnergy == NULL,
		"It is not allowed to set the common energy function more than once.");
		commonEnergy = energy;
	}

	void
	S_LP_PivotM_StructCUBNorm::setCommonSequenceString(
	 const std::string& seqStr) {
		assertbiu ( commonSeqStr.empty(),
		"It is not allowed to set the common sequence string more than once.");
		commonSeqStr = seqStr;
	}

	S_LP_PivotM_StructCUBNorm* 
	S_LP_PivotM_StructCUBNorm::clone() const {
		return new S_LP_PivotM_StructCUBNorm(*this);
	}

	CState*
	S_LP_PivotM_StructCUBNorm::compressedClone() const {
		return new CS_LP_PivotM_StructCUBNorm(
		 commonCUBLattice.parseMoveString(normalize()));
	}

	void
	S_LP_PivotM_StructCUBNorm::compress(CState& cState) const {
		dynamic_cast<CS_LP_PivotM_StructCUBNorm&>(cState)
		 = CS_LP_PivotM_StructCUBNorm(
			commonCUBLattice.parseMoveString(normalize()));
	}

	State::NeighborListPtr 
	S_LP_PivotM_StructCUBNorm::getNeighborList() const {
		return NeighborListPtr(new NeighborList(this));
	}

	State::NeighborListPtr
	S_LP_PivotM_StructCUBNorm::getRandomNeighborList()
				const {
		return NeighborListPtr(new RandomNeighborList(this));
	}

	// NeighborList
	S_LP_PivotM_StructCUBNorm::NeighborList::NeighborList(
	 			const S_LP_PivotM_StructCUBNorm* _origin) 
	 : S_LP_PivotM::NeighborList(_origin), origin(_origin)
	{
	}

	State* 
	S_LP_PivotM_StructCUBNorm::NeighborList::first(
		      State::NeighborList::ItState** itstate) const {
		if (*itstate != NULL) delete *itstate;

		ItState* myitstate = new ItState();
		*itstate = myitstate;

		S_LP_PivotM_StructCUBNorm* latProtState = NULL;

		PivotMove firstMove = origin->firstPivotMove();

		if (firstMove != PivotMove(0,INVALID_INDEX)) {
			latProtState = new S_LP_PivotM_StructCUBNorm(*origin);
			latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
			 firstMove.first, firstMove.second);
			myitstate->lastMove = firstMove;
				// if applied pivot move generates an invalid neighbor,
				// call next(..) to iterate neighbors until a VALID one is
				// found which will be returned
			if (!(latProtState->latProt.isValid())) {
				return next(*itstate, latProtState);
			}
		}

		return latProtState;
	}

	State* 
	S_LP_PivotM_StructCUBNorm::NeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const {
		assertbiu(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);

			// find next move
		PivotMove nextMove = origin->nextPivotMove(myitstate->lastMove);
			// no more moves possible
		if (nextMove == PivotMove(0,INVALID_INDEX)) {
			return NULL;
		}
		else {
			S_LP_PivotM_StructCUBNorm* latProtState =
			 dynamic_cast<S_LP_PivotM_StructCUBNorm*>(elem);
				// set latProtState to the state of origin of the neighbors
			latProtState->latProt = origin->latProt;
				// perform next move
			latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
			 nextMove.first, nextMove.second);
			myitstate->lastMove = nextMove;
				// if applied pivot move generates an invalid neighbor,
				// call next(..) to iterate neighbors until a VALID one is
				// found which will be returned
			if (!(latProtState->latProt.isValid())) {
				return next(itstate, latProtState);
			}
			return latProtState;
		}
	}

	// RandomNeighborList
	S_LP_PivotM_StructCUBNorm::RandomNeighborList::RandomNeighborList(
	 		const S_LP_PivotM_StructCUBNorm* _origin)
	 : S_LP_PivotM::RandomNeighborList(_origin),
	   origin(_origin) 
	{
	}

	State* 
	S_LP_PivotM_StructCUBNorm::RandomNeighborList::first(
		      State::NeighborList::ItState** itstate) const {
		if (*itstate != NULL) delete *itstate;

		*itstate = new ItState();
		S_LP_PivotM_StructCUBNorm* latProtState = 
		 new S_LP_PivotM_StructCUBNorm(*origin);

		return next(*itstate, latProtState);
	}

	State* 
	S_LP_PivotM_StructCUBNorm::RandomNeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const {
		assertbiu(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);
		S_LP_PivotM_StructCUBNorm* latProtState =
		 dynamic_cast<S_LP_PivotM_StructCUBNorm*>(elem);

		PivotMove nextMove;

			// try to find a valid move at random
		if (myitstate->switchModeValue > 0) {
				// position and index of the next pivot move
			size_t pos, moveIndex;
			bool foundValidMove = false;

			while (!foundValidMove) {
					// move position has to be in [1, structure length)
				pos = (random() % (origin->latProt.getStructureRef().size()-1))+1;
					// moveIndex has to be in [0, number of available moves)
				moveIndex = random() % origin->pivotMoveSet.getMoveNumber();

				nextMove = PivotMove(pos, moveIndex);

					// move has already been chosen
				if (!myitstate->chosenMoves.insert(nextMove).second) {
					continue;
				}
					// move hasn't been chosen yet
				else {
						// set latProtState to the state of origin of the
						// neighbors
					latProtState->latProt = origin->latProt;
					latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
					 nextMove.first, nextMove.second);
						// if applied pivot move generates an invalid neighbor,
						// iterate neighbors until a VALID one is found which
						// will be returned
					foundValidMove = latProtState->latProt.isValid();
					if (foundValidMove) {
						myitstate->switchModeValue--;
					}
				}	
			}
				// enough valid neighbors seen, switching mode to complete
				// valid neighbor enumeration
			if (myitstate->switchModeValue == 0) {
				switchMode(myitstate);
			}
		}
			// chose one of the precalculated valid moves 
		else {
			size_t n = myitstate->unchosenMoves.size();
			if (n == 0) {
				return NULL;
			}
			else {

// @TODO : Rekursion rausnehmen !!!

				size_t i = random() % n;
					// set latProtState to the state of origin of the neighbors
				latProtState->latProt = origin->latProt;
				nextMove = myitstate->unchosenMoves[i];
				latProtState->pivotMoveSet.applyMoveInPlace(
									&latProtState->latProt,
									nextMove.first, nextMove.second);
				myitstate->unchosenMoves[i] = myitstate->unchosenMoves[n-1];
				myitstate->unchosenMoves.resize(n-1);
					// if applied pivot move generates an invalid neighbor,
					// call next(..) to iterate neighbors until a VALID one is
					// found which will be returned
				if (!(latProtState->latProt.isValid())) {
					return next(itstate, latProtState);
				}
			}
		}
		return latProtState;
	}

	State* 
	S_LP_PivotM_StructCUBNorm::getRandomNeighbor(State* inPlaceNeigh) const {
		S_LP_PivotM_StructCUBNorm* neigh = NULL;
			// copy this state
		if (inPlaceNeigh != NULL) {
			neigh = dynamic_cast<S_LP_PivotM_StructCUBNorm*>(inPlaceNeigh);
			assertbiu(neigh != NULL, "inPlaceNeigh is no S_LP_PivotM"); 
			(*neigh) = (*this); 
		} else {
			neigh = new S_LP_PivotM_StructCUBNorm(*this);
		}

			// position and index of the next pivot move
		size_t pos, moveIndex;

		while (true) {
				// move position has to be in [1, structure length)
			pos = (random() % (latProt.getStructureRef().size()-1))+1;
				// moveIndex has to be in [0, number of available moves)
			moveIndex = random() % pivotMoveSet.getMoveNumber();

				// apply change
			pivotMoveSet.applyMoveInPlace(&neigh->latProt, pos, moveIndex);
				// if applied pivot move generates an invalid neighbor,
				// iterate neighbors until a VALID one is found which
				// will be returned
			if(neigh->latProt.isValid()) {
				return neigh;
			} else {
				// restore lattice protein
				pivotMoveSet.undoLastMove(&neigh->latProt);
			}
		}		
		return NULL;
	}

} // namespace ell
