#ifndef RNAFREEENERGYSINGLEMOVESTRUCTSTATE_HH_
#define RNAFREEENERGYSINGLEMOVESTRUCTSTATE_HH_


#include "S_RNAfe_SingleM.hh"

namespace ell
{
		/**
		 * This class generates neighbors in the energy landscape by
		 * adding or removing one bond to the RNA structure.
		 * All RNA structures of this class have a common RNA sequence.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class S_RNAfe_SingleM_Struct : public S_RNAfe_SingleM
	{
	protected:
		class NeighborList : public S_RNAfe_SingleM::NeighborList
		{
		protected:
				//! the state of origin of the neighbors
			const S_RNAfe_SingleM_Struct* const origin;
			
		public:
			NeighborList(const S_RNAfe_SingleM_Struct* _origin);

				/*! Returns a pointer to the first element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no neighbors exist.
				 */
			virtual State* first(State::NeighborList::ItState** itstate) const;
						 
				/*! Returns a pointer to the next element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no more neighbors exist.
				 */
			virtual State* next(	State::NeighborList::ItState* itstate, 
									State* elem) const;
		};

		class RandomNeighborList : public S_RNAfe_SingleM::
											RandomNeighborList
		{
		protected:
				//! the state of origin of the neighbors
			const S_RNAfe_SingleM_Struct* const origin;

		public:
			RandomNeighborList(const S_RNAfe_SingleM_Struct* _origin);

				/*! Returns a pointer to a random element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no neighbors exist.
				 */
			virtual State* first(State::NeighborList::ItState** itstate) const;
						 
				/*! Returns a pointer to the next random element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no more neighbors exist.
				 */
			virtual State* next(	State::NeighborList::ItState* itstate, 
									State* elem) const;
		};

			//! common RNA sequence string
		static std::string				commonRnaSeqStr;

	public:
			 /*! The common RNA sequence string has to be set before
			  *  calling the constructor.
			 */
		S_RNAfe_SingleM_Struct(const std::string& rnaStructBracketNotStr);
				
		S_RNAfe_SingleM_Struct( 
						const S_RNAfe_SingleM_Struct& rnaFESMS);
						
		virtual ~S_RNAfe_SingleM_Struct();

			/*! Sets common RNA sequence string.
			 *
			 *  Has to be called before initialisation of an
			 *  S_RNAfe_SingleM_Struct object and
			 *  has to be called only once.
			 */
		static void		setCommonSequenceString(const std::string& seqStr);

			/*! Returns the common RNA sequence string.
			 */
		static std::string	getCommonSequenceString() {
			return commonRnaSeqStr;
		}

	// abstract functions (State)
			/*! Returns a pointer to a clone of the current state. */
		virtual S_RNAfe_SingleM_Struct*	clone() const;
		
		virtual CState* compressedClone() const;

		virtual void	compress(CState& cState) const;

		virtual NeighborListPtr getNeighborList() const;
		
		virtual NeighborListPtr getRandomNeighborList() const;

		virtual State*	getRandomNeighbor(State* inPlaceNeigh = NULL) const;

	};

} // namespace ell

#endif /*RNAFREEENERGYSINGLEMOVESTRUCTSTATE_HH_*/
