#ifndef WC_LISTINGCOMPR_HH_
#define WC_LISTINGCOMPR_HH_

#include "WalkCollector.hh"
#include "CState.hh"
#include <vector>

namespace ell 
{
	
	/*!
	 * This subclass of of WalkCollector keeps a list of all accepted States.
	 * 
	 * @author Daniel Maticzka
	 */
	class WC_ListingCompr : public WalkCollector
	{
	private:
	
		State* lastState;
		std::vector<CState*> vec;
		
	public:
	
		WC_ListingCompr();
		~WC_ListingCompr();

		/*!
		 * This function is used to track all accepted intermediate States
		 * visited by ell::localMin search.
		 */
		virtual void setAccepted(const State& s);
		
		/*!
		 * This function is used to track all States that are visited 
		 * but not accepted by ell::localMin search.
		 */
		virtual void setChecked(const State& s);
		
		/*!
		 * Returns last State accepted by ell::localMin search.
		 */
		virtual State* getLastState() const;
		
		/*!
		 * Returns number of accepted States.
		 */
		virtual int getAcceptedCount() const;
	
		/*!
		 * Returns a list of all accepted States.
		 */
		std::vector<CState*> getAcceptedList() const;
	};
} // namespace ell

#endif /*WC_LISTINGCOMPR_HH_*/
