#include <iostream>
#include <ell/S_RNAfe_SingleM.hh>
#include <time.h>
#include <fstream>

int main(int argc, char **argv) {

	using namespace ell;

	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : S_RNAfe_SingleM\n"
				<<"===========================================" <<std::endl;
				
		// erzeugen	
	srandom( (unsigned) time(NULL) );

		std::string seqStr = "GGCAAAGUCAAAGAU";
	std::cout	<<"=> " <<seqStr <<" = seqStr" <<std::endl;
		std::string structStr = "..(...).(...)..";
	std::cout	<<"=> " <<structStr <<" = structStr" <<std::endl;
	std::cout	<<"=> create ell::S_RNAfe_SingleM(seqStr,"
				<<" structStr)" <<std::endl;
	ell::S_RNAfe_SingleM rnaStruct(seqStr, structStr);

		// getStateStringRepresentation()
	std::cout	<<"->getStateStringRepresentation()  = "
				<<rnaStruct.getStateStringRepresentation()
				<<std::endl;

		// getFitness()
	std::cout	<<"->getFitness()  = "
				<<(rnaStruct.getFitness())
				<<std::endl;

	ell::State* rnaStruct2 = rnaStruct.clone();

		// clone()  ==
	std::cout	<<"->(*this == *(this->clone()))  = "
				<<(rnaStruct == *rnaStruct2 ? "true":"false")
				<<std::endl;

		// clone()  !=
	std::cout	<<"->(*this != *(this->clone()))  = "
				<<(rnaStruct != *rnaStruct2 ? "true":"false")
				<<std::endl;

	int num = 0;
	
		// getNeighborList()
	ell::State::NeighborListPtr nl = rnaStruct.getNeighborList();
	std::set<std::string> neighs, randneighs;
	std::set<std::string>::const_iterator setIt;
	std::cout	<<"->getNeighborList()  ==> now sorted" <<std::endl;
	for(ell::State::NeighborList::Iterator it = nl->begin();
		 it != nl->end(); ++it) {
		neighs.insert(it->getStateStringRepresentation());
	}
	for(setIt = neighs.begin(); setIt != neighs.end(); setIt++) {
		std::cout <<" " <<++num <<"\t: " <<*setIt <<std::endl;
	}

		// getRandomNeighborList()
	num = 0;
	nl = rnaStruct.getRandomNeighborList();
	std::cout	<<"->getRandomNeighborList() ==> now sorted" <<std::endl;
	for(ell::State::NeighborList::Iterator it = nl->begin();
		 it != nl->end(); ++it) {
		randneighs.insert(it->getStateStringRepresentation());
	}
	for(setIt = randneighs.begin(); setIt != randneighs.end(); setIt++) {
		std::cout <<" " <<++num <<"\t: " <<*setIt <<std::endl;
	}

		// getMinimalDistance(..)
	std::cout	<<"Distance to GGCAAAGUCAAAGAU (.((...)....).): "
				<<rnaStruct.getMinimalDistance(
					ell::S_RNAfe_SingleM(	"GGCAAAGUCAAAGAU", 
														"(.((...)....).)"))
				<<std::endl;

	ell::CState* cRnaStruct = rnaStruct.compressedClone();
	ell::CState* cRnaStruct2 = rnaStruct2->compressedClone();
	rnaStruct.compress(*cRnaStruct2);

		// compressedClone()  ==   compress(..)
	std::cout	<<"=>this.compress(cRNA)" <<std::endl;
	std::cout	<<"->(*(this.compressedClone()) == cRna)  = "
				<<(*cRnaStruct == *cRnaStruct2 ? "true":"false")
				<<std::endl;

		// compressedClone()  !=   compress(..)
	std::cout	<<"->(*(this.compressedClone()) != cRna)  = "
				<<(*cRnaStruct != *cRnaStruct2 ? "true":"false")
				<<std::endl;

	ell::CState* cRnaStruct4 = cRnaStruct2->clone();

		// cRna  ==  *(cRna.clone())
	std::cout	<<"->(cRna == *(cRna.clone()))  = "
				<<(*cRnaStruct2 == *cRnaStruct4 ? "true":"false")
				<<std::endl;

		// getCStateStringRepresentation()
	std::cout	<<"->(this.compressedClone()->getCStateStringRepresentation()"
				<<"  = "
				<<cRnaStruct->getCStateStringRepresentation()
				<<std::endl;

		// getUncomprStateStringRepresentation()
	std::cout	<<"->getUncomprStateStringRepresentation()  = "
				<<cRnaStruct->getUncomprStateStringRepresentation()
				<<std::endl;

	ell::State* rnaStruct3 = cRnaStruct->uncompressedClone();

		// uncompressedClone()
	std::cout	<<"->(*(this.compressedClone()->uncompressedClone()) == "
				<<"*(this))  = "
				<<(*rnaStruct3 == rnaStruct ? "true":"false")
				<<std::endl;

	cRnaStruct->uncompress(*rnaStruct2);

		// uncompress(..)
	std::cout	<<"=>this.compressedClone()->uncompress(rna)" <<std::endl;
	std::cout	<<"->(*(this) == rna)  = "
				<<(*rnaStruct2 == rnaStruct ? "true":"false")
				<<std::endl;

	std::ofstream out("out.bin");
	if (!out) {
		std::cerr<<"out.bin couldn't be opened!"<<std::endl;
		exit(-1);
	}
	cRnaStruct2->writeBinaryOutput(out);
	out.close();
	std::ifstream in("out.bin");
	if (!in) {
		std::cerr<<"out.bin couldn't be opened!"<<std::endl;
		exit(-1);
	}
	ell::CState* cRnaStruct3 = cRnaStruct2->readBinaryInputClone(in);
	in.close();

		// writeBinaryOutput(..), readBinaryInputClone(..)
	std::cout	<<"=>std::ofstream out(\"out.bin\")" <<std::endl;
	std::cout	<<"=>cRna.writeBinaryOutput(out)" <<std::endl;
	std::cout	<<"=>std::ifstream in(\"out.bin\")" <<std::endl;
	std::cout	<<"=>cRna2 = cRna.readBinaryInputClone(in)" <<std::endl;
	std::cout	<<"->(*cRna == *cRna2)  = "
				<<(*cRnaStruct2 == *cRnaStruct3 ? "true":"false")
				<<std::endl;

	delete rnaStruct2;
	delete rnaStruct3;
	delete cRnaStruct;
	delete cRnaStruct2;
	delete cRnaStruct3;
	delete cRnaStruct4;
	
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
