#include "StateAcceptor.hh"
#include "WalkCollector.hh"
#include "State.hh"
#include <iostream>

using namespace ell;

/*!
 * This is a dummy implementation of a State class. The only functionality
 * implemented is setting an id and energy in the constructor and retrieving it via
 * getStateStringRepresentation and getEnergy.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
	public:
	
	std::string id;
	double energy;

	DummyState(std::string _id, double _energy)
	{
		id = _id; 
		energy = _energy;
	}
	
	~DummyState()
	{}
	
	bool operator== (const State& state2) const {return true;}
	bool operator!= (const State& state2) const {return true;}

	int	getFitness() const 
	{
		return 0;
	}
	
	double getEnergy() const 
	{
		return energy;
	}
	
	unsigned int getMinimalDistance(const State& state2) const 
	{
		return 0;
	}
	
	State* clone() const
	{
		return new DummyState(id,energy);
	}
	
	std::string	getStateStringRepresentation() const 
	{
		return id;
	}

	CState*	compressedClone() const
	{
		return NULL;
	}

	void compress(CState& cState) const {}

	NeighborListPtr getNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	NeighborListPtr getRandomNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const
	{
		return NULL;
	}
};

bool
StateAcceptor::accept(WalkCollector* wc, const State& succ)
{
	return true;
};

int main(int argc, char** argv)
{
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : SA_All\n"
				<<"===========================================" <<std::endl;
	std::cout	<< "=> creating WalkCollector" << std::endl;
	WalkCollector* wc = new WalkCollector();
	std::cout	<< "=> creating State" << std::endl;
	State* s = new DummyState("State",0);
	std::cout	<< "=> creating SA_All" << std::endl;
	StateAcceptor* sa = new SA_All();
	
	std::cout	<< "->accept = " << sa->accept(wc,*s) << std::endl;
	
	delete sa;
	delete wc;
	delete s;
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;

	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : SA_E_Gr\n"
				<<"===========================================" <<std::endl;
	std::cout	<< "=> creating DummyState(\"wcstate\", 5)" << std::endl;
	State* wcstate = new DummyState("wcstate", 5);
	std::cout	<< "=> creating WalkCollector" << std::endl;
	wc = new WalkCollector();
	std::cout	<< "->setAccepted(wcstate)" << std::endl;
	wc->setAccepted(*wcstate);
	
	std::cout	<< "=> creating DummyState(\"State\", 4)" << std::endl;
	s = new DummyState("State", 4);
	std::cout	<< "=> creating SA_E_Gr" << std::endl;
	sa = new SA_E_Gr();
	std::cout	<< "->accept = " << sa->accept(wc,*s) << std::endl;
	delete sa;
	delete s;
	
	std::cout	<< "=> creating DummyState(\"State\", 6)" << std::endl;
	s = new DummyState("State", 6);
	std::cout	<< "=> creating SA_E_Gr" << std::endl;
	sa = new SA_E_Gr();
	std::cout	<< "->accept = " << sa->accept(wc,*s) << std::endl;
	delete sa;
	delete s;

	delete wc;
	delete wcstate;
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : SA_E_Le\n"
				<<"===========================================" <<std::endl;
	std::cout	<< "=> creating DummyState(\"wcstate\", 5)" << std::endl;
	wcstate = new DummyState("wcstate", 5);
	std::cout	<< "=> creating WalkCollector" << std::endl;
	wc = new WalkCollector();
	std::cout	<< "->setAccepted(wcstate)" << std::endl;
	wc->setAccepted(*wcstate);
	
	std::cout	<< "=> creating DummyState(\"State\", 4)" << std::endl;
	s = new DummyState("State", 4);
	std::cout	<< "=> creating SA_E_Le" << std::endl;
	sa = new SA_E_Le();
	std::cout	<< "->accept = " << sa->accept(wc,*s) << std::endl;
	delete sa;
	delete s;
	
	std::cout	<< "=> creating DummyState(\"State\", 6)" << std::endl;
	s = new DummyState("State", 6);
	std::cout	<< "=> creating SA_E_Le" << std::endl;
	sa = new SA_E_Le();
	std::cout	<< "->accept = " << sa->accept(wc,*s) << std::endl;
	delete sa;
	delete s;

	delete wc;
	delete wcstate;
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : SA_E_Eq\n"
				<<"===========================================" <<std::endl;
	std::cout	<< "=> creating DummyState(\"wcstate\", 5)" << std::endl;
	wcstate = new DummyState("wcstate", 5);
	std::cout	<< "=> creating WalkCollector" << std::endl;
	wc = new WalkCollector();
	std::cout	<< "->setAccepted(wcstate)" << std::endl;
	wc->setAccepted(*wcstate);
	
	std::cout	<< "=> creating DummyState(\"State\", 4)" << std::endl;
	s = new DummyState("State", 4);
	std::cout	<< "=> creating SA_E_Eq" << std::endl;
	sa = new SA_E_Eq();
	std::cout	<< "->accept = " << sa->accept(wc,*s) << std::endl;
	delete sa;
	delete s;
	
	std::cout	<< "=> creating DummyState(\"State\", 5)" << std::endl;
	s = new DummyState("State", 5);
	std::cout	<< "=> creating SA_E_Eq" << std::endl;
	sa = new SA_E_Eq();
	std::cout	<< "->accept = " << sa->accept(wc,*s) << std::endl;
	delete sa;
	delete s;

	delete wc;
	delete wcstate;
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : SA_E_GrEq\n"
				<<"===========================================" <<std::endl;
	std::cout	<< "=> creating DummyState(\"wcstate\", 5)" << std::endl;
	wcstate = new DummyState("wcstate", 5);
	std::cout	<< "=> creating WalkCollector" << std::endl;
	wc = new WalkCollector();
	std::cout	<< "->setAccepted(wcstate)" << std::endl;
	wc->setAccepted(*wcstate);
	
	std::cout	<< "=> creating DummyState(\"State\", 4)" << std::endl;
	s = new DummyState("State", 4);
	std::cout	<< "=> creating SA_E_GrEq" << std::endl;
	sa = new SA_E_GrEq();
	std::cout	<< "->accept = " << sa->accept(wc,*s) << std::endl;
	delete sa;
	delete s;
	
	std::cout	<< "=> creating DummyState(\"State\", 5)" << std::endl;
	s = new DummyState("State", 5);
	std::cout	<< "=> creating SA_E_GrEq" << std::endl;
	sa = new SA_E_GrEq();
	std::cout	<< "->accept = " << sa->accept(wc,*s) << std::endl;
	delete sa;
	delete s;

	delete wc;
	delete wcstate;
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : SA_E_LeEq\n"
				<<"===========================================" <<std::endl;
	std::cout	<< "=> creating DummyState(\"wcstate\", 5)" << std::endl;
	wcstate = new DummyState("wcstate", 5);
	std::cout	<< "=> creating WalkCollector" << std::endl;
	wc = new WalkCollector();
	std::cout	<< "->setAccepted(wcstate)" << std::endl;
	wc->setAccepted(*wcstate);
	
	std::cout	<< "=> creating DummyState(\"State\", 6)" << std::endl;
	s = new DummyState("State", 6);
	std::cout	<< "=> creating SA_E_LeEq" << std::endl;
	sa = new SA_E_LeEq();
	std::cout	<< "->accept = " << sa->accept(wc,*s) << std::endl;
	delete sa;
	delete s;
	
	std::cout	<< "=> creating DummyState(\"State\", 5)" << std::endl;
	s = new DummyState("State", 5);
	std::cout	<< "=> creating SA_E_LeEq" << std::endl;
	sa = new SA_E_LeEq();
	std::cout	<< "->accept = " << sa->accept(wc,*s) << std::endl;
	delete sa;
	delete s;

	delete wc;
	delete wcstate;
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
