#include "WC_Listing.hh"
#include "State.hh"
#include <iostream>
#include <vector>

using namespace ell;

/*!
 * This is a dummy implementation of a State class. The only functionality
 * implemented is setting an id in the constructor and retrieving it via
 * getStateStringRepresentation.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
	public:
	
	std::string id;

	DummyState(std::string _id)
	{
		id = _id; 
	}
	
	~DummyState() {}
	
	bool operator== (const State& state2) const {return true;}
	bool operator!= (const State& state2) const {return true;}

	int	getFitness() const 
	{
		return 0;
	}
	
	double getEnergy() const 
	{
		return 0;
	}
	
	unsigned int getMinimalDistance(const State& state2) const 
	{
		return 0;
	}
	
	State* clone() const
	{
		return new DummyState(id);
	}
	
	std::string	getStateStringRepresentation() const 
	{
		return id;
	}

	CState*	compressedClone() const
	{
		return NULL;
	}

	void compress(CState& cState) const {}

	NeighborListPtr getNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	NeighborListPtr getRandomNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const
	{
		return NULL;
	}
};

int main(int argc, char** argv)
{
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : WC_History\n"
				<<"===========================================" <<std::endl;
	
	// create some States
	std::cout 	<< "=> creating DummyStates s1,s2,s3" << std::endl;
	State* s1 = new DummyState("State 1");
	State* s2 = new DummyState("State 2");
	State* s3 = new DummyState("State 3");
	
	// test WC_Listing
	std::cout 	<< "=> creating ell::WC_Listing" << std::endl;
	WalkCollector* wc = new WC_Listing();
	
	// check in States
	std::cout << "->setAccepted(*s1)" << std::endl;
	wc->setAccepted(*s1);
	std::cout << "->setAccepted(*s2)" << std::endl;
	wc->setAccepted(*s2);
	std::cout << "->setChecked(*s3)" << std::endl;
	wc->setChecked(*s3);
	
	// retrieve number of states
	int count = wc->getAcceptedCount();
	std::cout 	<< "->getAcceptedCount = " << count << std::endl;
				
	// try to retrieve States
	WC_Listing* wch = (WC_Listing*) wc;
	std::cout 	<< "->getAcceptedList = " << std::endl;
	std::vector<State*> vec = wch->getAcceptedList();
	std::vector<State*>::iterator iter;
  	for( iter = vec.begin(); iter != vec.end(); iter++ ) 
  	{
    	std::cout 	<< "   " 
    				<< (*iter)->getStateStringRepresentation() 
    				<< std::endl;
  	}
	
  	delete wc;
	
	delete s1;
	delete s2;
	delete s3;
	
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
