#include "WC_ListingCompr.hh"
#include "State.hh"
#include "CState.hh"
#include <iostream>
#include <vector>
#include <string>

using namespace ell;

/*!
 * This is a dummy implementation of a State class. The only functionality
 * implemented is setting an id in the constructor and retrieving it via
 * getStateStringRepresentation.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
public:
	std::string id;

	DummyState(std::string _id) { id = _id;	}
	~DummyState() {}
	
	bool operator== (const State& state2) const;
	bool operator!= (const State& state2) const;
	int	getFitness() const;
	double getEnergy() const;
	unsigned int getMinimalDistance(const State& state2) const;
	State* clone() const;
	std::string	getStateStringRepresentation() const;
	CState*	compressedClone() const;
	void compress(CState& cState) const;
	NeighborListPtr getNeighborList() const;
	NeighborListPtr getRandomNeighborList() const; 
	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const;
	
};

/*!
 * This is a dummy implementation of a CState class.
 * 
 * @author Daniel Maticzka
 */
class DummyCState : public CState
{
public:
	std::string id;

	DummyCState(std::string _id) { id = _id; }
	~DummyCState() {}
	
	bool operator==(const ell::CState&) const { return true; }
	bool operator!=(const ell::CState&) const { return true; }
	CState* clone() const { return new DummyCState(id); }
	std::string getCStateStringRepresentation() const { return id; }
	std::string getUncomprStateStringRepresentation() const { return id; }
	State* uncompressedClone() const { return new DummyState(id); }
	void uncompress(ell::State&) const { return; }
	CState* readBinaryInputClone(std::ifstream&) const { return NULL; }
	void writeBinaryOutput(std::ofstream&) const { return;}
};

bool DummyState::operator== (const State& state2) const {return true;}
bool DummyState::operator!= (const State& state2) const {return true;}
int	DummyState::getFitness() const { return 0; }
double DummyState::getEnergy() const { return 0; }
unsigned int DummyState::getMinimalDistance(const State& state2) const { return 0; }
State* DummyState::clone() const { return new DummyState(id); }
std::string	DummyState::getStateStringRepresentation() const { return id; }
CState*	DummyState::compressedClone() const	{ return new DummyCState(id); }
void DummyState::compress(CState& cState) const {}
State::NeighborListPtr DummyState::getNeighborList() const	{ return *(new NeighborListPtr); }
State::NeighborListPtr DummyState::getRandomNeighborList() const 
	{ return *(new NeighborListPtr); }
State* DummyState::getRandomNeighbor(State* inPlaceNeigh) const
	{ return NULL; }

int main(int argc, char** argv)
{
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : WC_History\n"
				<<"===========================================" <<std::endl;
	
	// create some States
	std::cout 	<< "=> creating DummyStates s1,s2,s3" << std::endl;
	State* s1 = new DummyState("State 1");
	State* s2 = new DummyState("State 2");
	State* s3 = new DummyState("State 3");
	
	// test WC_ListingCompr
	std::cout 	<< "=> creating ell::WC_ListingCompr" << std::endl;
	WalkCollector* wc = new WC_ListingCompr();
	
	// check in States
	std::cout << "->setAccepted(*s1)" << std::endl;
	wc->setAccepted(*s1);
	std::cout << "->setAccepted(*s2)" << std::endl;
	wc->setAccepted(*s2);
	std::cout << "->setChecked(*s3)" << std::endl;
	wc->setChecked(*s3);
	
	// retrieve number of states
	int count = wc->getAcceptedCount();
	std::cout 	<< "->getAcceptedCount = " << count << std::endl;
				
	// try to retrieve States
	WC_ListingCompr* wch = (WC_ListingCompr*) wc;
	std::cout 	<< "->getAcceptedList = " << std::endl;
	std::vector<CState*> vec = wch->getAcceptedList();
	std::vector<CState*>::iterator iter;
  	for( iter = vec.begin(); iter != vec.end(); iter++ ) 
  	{
    	std::cout 	<< "   " 
    				<< (*iter)->getCStateStringRepresentation() 
    				<< std::endl;
  	}
	
  	delete wc;
	
	delete s1;
	delete s2;
	delete s3;
	
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
